// Backbone Server
// Nate Hunzaker
//
// Adds a module for server side interaction within Backbone using Express


//-- Requirements ------------------------------------------------------//

// Side Dishes
require('colors');

var _          = require('underscore'),
    express    = require('express'),
    ejs        = require('ejs'),
    fs         = require('fs');

// The Main Course
module.exports.Backbone = Backbone = require('backbone');


//-- Backbone Server ---------------------------------------------------//
Backbone.Server = function (options) {
    
    //-- Default Values ------------------------------------------------//
    var settings = {
        'database'       : false,
        'express'        : express.createServer(),
        'port'           : 8000,
        'public'         : 'public',
        'routes'         : {},
        'socketio'       : false,
        'views'          : 'views',
        'view engine'    : 'ejs'                // We use EJS as the default, simply because it has good synergy with natively with backbone
    };

    // Add Default Overides
    _.extend(this, settings, options);

    // Set everything else up
    this._setup();
};


// Backbone Server Functionality
_.extend(Backbone.Server.prototype, Backbone.Events, {
    
    //-- Modules  ------------------------------------------------------//

    // A nifty helper to load all files within a directory
    // Note : This is basically a glorified "require all in directory" function.
    //        Should further releases of node include such functionality, it will
    //        be deprecated.
    load: function(module) {
        
        var dir = require.main.filename.split("/").slice(0, require.main.filename.split("/").length - 1).join("/"),
            files = fs.readdirSync( dir + "/" + module, 'utf-8');
        
        files.forEach(function(file) {
          require( dir + "/" + module + "/" + file );
        });

    },
    
    //-- Standard CRUD Routing Methods  --------------------------------//

    route: function (method, path, action) {
        this.routes[path] = [method, action];
    },

    get: function(path, action) {
        this.route('get', path, action);
    },

    post: function(path, action) {
        this.route('post', path, action);
    },

    put: function(path, action) {
        this.route('put', path, action);
    },

    delete: function(path, action) {
        this.route('delete', path, action);
    },

    _setup: function() {
        
        var server = this;

        // Set static content folder
        this.express.use(express.static(this.public));
        
        // Set the view engine
        this.express.set('view engine', this['view engine']);    


        //-- SocketIO Methods  -----------------------------------------//
        
        if ( this.socketio ) {

            this.io = require('socket.io').listen(this.express);
            
            // A syntax sugar for socketIO
            this.on = function(name, action) {
                this.io.sockets.on(name, action);
            };
            
            this.emit = function(name, values) {
                this.io.sockets.emit(name, values);
            };

            this.events = function(events) {
                
                for (var e in events) {
                    this.bind(e, events[e]);
                }

            };

            console.log("Socket.IO engaged.".blue);
        }

        //-- Mongo Methods  --------------------------------------------//
            
        if (this.database.adapter === 'mongo') {
            
            var Schema = require('./backbone-mongo').Schema;
            
            require('./backbone-mongo').setup(this.database);

            this.Datamodel = {
                mongoose: require('./backbone-mongo').mongoose
            };

            this.Datamodel.register = function(name, schema) {
                
                if (!name) {
                    throw new Error("\nPlease provide a name for this MongoDB model.\n".red.bold)
                }
                if (!schema) {
                    throw new Error("\nPlease provide a schema for ".red.bold + name.red.bold + ".\n".red.bold);
                } 

                var new_schema = new Schema(schema);

                server.Datamodel[name] = mongoose.model(name, new_schema);
            };
        
        } 
    },

    //-- Server Methods ------------------------------------------------//

    start: function() {   

        // Add routes
        for (var i in this.routes) {
            
            var method = this.routes[i][0],
                action = this.routes[i][1];
            
            this.express[method](i, action);
        }
        
        // Add a route to load backbone:
        this.express.get('/backbone-server/backbone.js', function(req, res) {
                        
            var underscorejs = fs.readFileSync(__dirname + '/node_modules/underscore/underscore-min.js', 'utf-8'),
                backbonejs   = fs.readFileSync(__dirname + '/node_modules/backbone/backbone.js', 'utf-8');

            res.contentType('text/javascript');
            res.send(underscorejs + "\n" + backbonejs);
        
        });
        
        this.express.listen(this.port);

        console.log("Backbone Server is listening on port ".blue + this.port.toString().yellow);
    }
    
});